<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the dashboard.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    Ctlt_Ensure_User_Member_Of_Root_Blog
 * @subpackage Ctlt_Ensure_User_Member_Of_Root_Blog/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, dashboard-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Ctlt_Ensure_User_Member_Of_Root_Blog
 * @subpackage Ctlt_Ensure_User_Member_Of_Root_Blog/includes
 * @author     Richard Tape <richard.tape@ubc.ca>
 */
class Ctlt_Ensure_User_Member_Of_Root_Blog {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Ctlt_Ensure_User_Member_Of_Root_Blog_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $ctlt_ensure_user_member_of_root_blog    The string used to uniquely identify this plugin.
	 */
	protected $ctlt_ensure_user_member_of_root_blog;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the Dashboard and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->ctlt_ensure_user_member_of_root_blog = 'ctlt-ensure-user-member-of-root-blog';
		$this->version = '1.0.0';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Ctlt_Ensure_User_Member_Of_Root_Blog_Loader. Orchestrates the hooks of the plugin.
	 * - Ctlt_Ensure_User_Member_Of_Root_Blog_i18n. Defines internationalization functionality.
	 * - Ctlt_Ensure_User_Member_Of_Root_Blog_Admin. Defines all hooks for the dashboard.
	 * - Ctlt_Ensure_User_Member_Of_Root_Blog_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-ctlt-ensure-user-member-of-root-blog-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-ctlt-ensure-user-member-of-root-blog-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the Dashboard.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-ctlt-ensure-user-member-of-root-blog-admin.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-ctlt-ensure-user-member-of-root-blog-public.php';

		$this->loader = new Ctlt_Ensure_User_Member_Of_Root_Blog_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Ctlt_Ensure_User_Member_Of_Root_Blog_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Ctlt_Ensure_User_Member_Of_Root_Blog_i18n();
		$plugin_i18n->set_domain( $this->get_ctlt_ensure_user_member_of_root_blog() );

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the dashboard functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Ctlt_Ensure_User_Member_Of_Root_Blog_Admin( $this->get_ctlt_ensure_user_member_of_root_blog(), $this->get_version() );

		// $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		// $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Ctlt_Ensure_User_Member_Of_Root_Blog_Public( $this->get_ctlt_ensure_user_member_of_root_blog(), $this->get_version() );

		// $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
		// $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
		// On create user, add the user to the root blog
		// On login, check that the logged in user is a member of the root blog and if not, add them
		// We have to encapsulate both of these in a loader, on plugins_loaded
		$this->loader->add_action( 'plugins_loaded', $plugin_public, 'plugins_loaded__setupRegisterAndLoginActions'  );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_ctlt_ensure_user_member_of_root_blog() {
		return $this->ctlt_ensure_user_member_of_root_blog;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Ctlt_Ensure_User_Member_Of_Root_Blog_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
