<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       http://example.com
 * @since      1.0.0
 *
 * @package    Ctlt_Ensure_User_Member_Of_Root_Blog
 * @subpackage Ctlt_Ensure_User_Member_Of_Root_Blog/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the dashboard-specific stylesheet and JavaScript.
 *
 * @package    Ctlt_Ensure_User_Member_Of_Root_Blog
 * @subpackage Ctlt_Ensure_User_Member_Of_Root_Blog/public
 * @author     Richard Tape <richard.tape@ubc.ca>
 */
class Ctlt_Ensure_User_Member_Of_Root_Blog_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $ctlt_ensure_user_member_of_root_blog    The ID of this plugin.
	 */
	private $ctlt_ensure_user_member_of_root_blog;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @var      string    $ctlt_ensure_user_member_of_root_blog       The name of the plugin.
	 * @var      string    $version    The version of this plugin.
	 */
	public function __construct( $ctlt_ensure_user_member_of_root_blog, $version ) {

		$this->ctlt_ensure_user_member_of_root_blog = $ctlt_ensure_user_member_of_root_blog;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Ctlt_Ensure_User_Member_Of_Root_Blog_Public_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Ctlt_Ensure_User_Member_Of_Root_Blog_Public_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		// wp_enqueue_style( $this->ctlt_ensure_user_member_of_root_blog, plugin_dir_url( __FILE__ ) . 'css/ctlt-ensure-user-member-of-root-blog-public.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		/**
		 * This function is provided for demonstration purposes only.
		 *
		 * An instance of this class should be passed to the run() function
		 * defined in Ctlt_Ensure_User_Member_Of_Root_Blog_Public_Loader as all of the hooks are defined
		 * in that particular class.
		 *
		 * The Ctlt_Ensure_User_Member_Of_Root_Blog_Public_Loader will then create the relationship
		 * between the defined hooks and the functions defined in this
		 * class.
		 */

		// wp_enqueue_script( $this->ctlt_ensure_user_member_of_root_blog, plugin_dir_url( __FILE__ ) . 'js/ctlt-ensure-user-member-of-root-blog-public.js', array( 'jquery' ), $this->version, false );

	}

	/**
	 * Set up the actions for new user register and login to add/detect if a user is attached to the root blog
	 *
	 * @author Richard Tape <@richardtape>
	 * @since 1.0
	 * @param null
	 * @return null
	 */

	public function plugins_loaded__setupRegisterAndLoginActions() {

		// Add new user to root blog
		add_action( 'wpmu_new_user', array( $this, 'wpmu_new_user__addNewUserToRootBlog' ), 5, 1 );
		add_action( 'cas_wpmu_new_user', array( $this, 'wpmu_new_user__addNewUserToRootBlog' ), 5, 1 );

		// On login, check that this user is attached to the root blog, if not add them
		add_action( 'wp_login', array( $this, 'wp_login__checkUserIsOnRootBlogAndAddIfNot' ), 2, 2 );
		add_action( 'cas_wp_login', array( $this, 'wp_login__checkUserIsOnRootBlogAndAddIfNot' ), 2, 2 );

		// If a user is attempting to access the root blog and they have been removed, re-add them and redirect
		add_action( 'admin_page_access_denied', array( $this, 'admin_page_access_denied__addUserToRootBlog' ), 5 );

	}/* plugins_loaded__setupRegisterAndLoginActions() */

	
	/**
	 * On user register, add them to the root blog
	 *
	 * @author Richard Tape <@richardtape>
	 * @since 1.0
	 * @param (int) $userID - The ID of the user just created
	 * @return null
	 */
	
	public function wpmu_new_user__addNewUserToRootBlog( $userID ) {

		// Need the root blog ID
		$blogID = static::getRootBlogID();

		// Go ahead and add
		static::addUserToBlog( $userID, $blogID );

	}/* wp_login__checkUserIsOnRootBlogAndAddIfNot() */

	/**
	 * When a user logs in, check that they are a member of the root blog. If not, add them as a subscriber
	 *
	 * @author Richard Tape <@richardtape>
	 * @since 1.0
	 * @param (string) $userLogin - The login name of the user
	 * @param (object) $user - The full user object
	 * @return null
	 */
	
	public function wp_login__checkUserIsOnRootBlogAndAddIfNot( $userLogin, $user ) {

		// Just use the UserID
		$userID = $user->ID;

		if ( !$userID ) {
			return false;
		}
		// Need the root blog ID
		$blogID = static::getRootBlogID();


		// Check if this user is already on the root blog
		if( static::userIsMemberOfBlog( $userID, $blogID ) ) {
			return;
		}

		// OK, they're not attached to the root blog, add them
		static::addUserToBlog( $userID, $blogID, 'subscriber' );

	}/* wp_login__checkUserIsOnRootBlogAndAddIfNot() */


	/**
	 * If a user is about to hit the access denied wp_die screen, we can hook in before they
	 * see the message and, if it's the root site's dashboard they are trying to access, then we
	 * can add them, and redirect
	 *
	 * @author Richard Tape <@richardtape>
	 * @since 1.0
	 * @param null
	 * @return null
	 */
	
	public function admin_page_access_denied__addUserToRootBlog() {

		if ( ! is_user_logged_in() || is_network_admin() ) {
			return;
		}

		// Check if we're looking at the root blog
		$currentBlogID = get_current_blog_id();
		$rootBlogID = static::getRootBlogID();

		if( $currentBlogID != $rootBlogID ) {
			return;
		}

		$userID = get_current_user_id();

		// Check if this user is already on this blog
		$blogs = get_blogs_of_user( $userID );

		if ( wp_list_filter( $blogs, array( 'userblog_id' => $currentBlogID ) ) ) {
			return;
		}

		// OK, user isn't a member of the root blog, add them
		static::addUserToBlog( $userID, $rootBlogID, 'subscriber' );

		// Now, in order to not see the wp_die() screen which is at the bottom of _access_denied_splash()
		// we need to do a wp_redirect back to ourself
		wp_redirect( admin_url( '', 'https' ) );
		exit();

	}/* admin_page_access_denied__addUserToRootBlog() */
	


	/**
	 * Check if a user ID is attached to a specified blog
	 *
	 * @author Richard Tape <@richardtape>
	 * @since 1.0
	 * @param (int)   $userID - The userID to check
	 * @param (int)   $blogID - The blog ID to see if $userID is a member
	 * @return (bool) - True if the user with an ID of $userID is already a member of the blog with an ID of $blogID, false otherwise
	 */

	public static function userIsMemberOfBlog( $userID = false, $blogID = false ) {

		// We user ID
		if ( !$userID ) {
			return false;
		}

		// If blogID is not passed, assume the root site
		if( !$blogID ) {
			$blogID = static::getRootBlogID();
		}

		// Sanitize
		$userID = intval( $userID );
		$blogID = intval( $blogID );

		// Check if sanitization failed
		if ( !$userID || !$blogID ) {
			return false;
		}

		// OK, now check if the user is a member of the blog
		return is_user_member_of_blog( $userID, $blogID );

	}/* userIsMemberOfBlog() */


	/**
	 * Add a user to a specifid blog with the specified role
	 *
	 * @author Richard Tape <@richardtape>
	 * @since 1.0
	 * @param (int)   $userID - The userID to add
	 * @param (int)   $blogID - The blog ID to add $userID to
	 * @param (string) $role   - The role which this user should be added
	 * @return (bool) - True if user is added, False otherwise
	 */

	public static function addUserToBlog( $userID = false, $blogID = false, $role = 'subscriber' ) {

		// We need a userID
		if ( !$userID ) {
			return false;
		}

		// If blogID is not passed, assume the root site
		if( !$blogID ) {
			$blogID = static::getRootBlogID();
		}

		// Sanitize
		$userID = intval( $userID );
		$blogID = intval( $blogID );
		$role = sanitize_text_field( $role );

		// Check we still have all 3 after sanitization
		if ( !$userID || !$blogID || !$role ) {
			return false;
		}

		// Check if the user is already a member of this blog
		if ( static::userIsMemberOfBlog( $userID, $blogID ) === true ) {
			return false;
		}

		// OK, add the user and if all goes well, say so
		if ( add_user_to_blog( $blogID, $userID, $role ) == true ) {
			return true;
		} else {
			return false;
		}

	}/* addUserToBlog() */


	/**
	 * Get the root blog ID
	 *
	 * @author Richard Tape <@richardtape>
	 * @since 1.0
	 * @param null
	 * @return (int) - $rootBlogID - the site ID of the root blog
	 */
	
	public static function getRootBlogID() {

		// If we have our custom define (mainly for CMS because sites.olt.ubc.ca has a redirect to cms.ubc.ca)
		if( defined( 'CTLT_MAIN_SITE_ID' ) ) {
			return CTLT_MAIN_SITE_ID;
		}

		// If this constant is set, use it - this will probably be for blogs
		if( defined( 'BLOG_ID_CURRENT_SITE' ) ) {
			return BLOG_ID_CURRENT_SITE;
		}

		// Assume 1, otherwise
		return 1;

	}/* getRootBlogID() */


}
